Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

function flushCmdLine(aCmdLine) {
  if (aCmdLine.length)
    aCmdLine.removeArguments(0, aCmdLine.length - 1);
}

function isDisabled() {
  var RDFService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
    .getService(Components.interfaces.nsIRDFService);
  var extension = RDFService
    .GetResource("urn:mozilla:item:{a069f3a6-e5c8-4d6c-a6b1-55114d145feb}");
  var userDisabledArc = RDFService
    .GetResource("http://www.mozilla.org/2004/em-rdf#userDisabled");
  return Components.classes["@mozilla.org/extensions/manager;1"]
    .getService(Components.interfaces.nsIExtensionManager).datasource
    .hasArcOut(extension, userDisabledArc);
}

function openWindow(aParent, aURL, aTarget, aFeatures, aArgs) {
  var args = Components.classes["@mozilla.org/supports-array;1"]
    .createInstance(Components.interfaces.nsICollection);
  while (aArgs && aArgs.length > 0) {
    var arg = aArgs.shift();
    var argstring = Components.classes["@mozilla.org/supports-string;1"]
      .createInstance(Components.interfaces.nsISupportsString);
    argstring.data = arg ? arg : "";
    args.AppendElement(argstring);
  }
  Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
      .getService(Components.interfaces.nsIWindowWatcher)
      .openWindow(aParent, aURL, aTarget, aFeatures, args);
}

function PrintPDFHandler() {}
PrintPDFHandler.prototype = {
  handle : function comp_hadle(aCmdLine) {
    var flagPos = aCmdLine.findFlag("printpdf", false);
    if (flagPos == -1)
      return;

    aCmdLine.preventDefault = true;

    if (flagPos == aCmdLine.length - 1) {
      dump("Missing output file name.\n");
      flushCmdLine(aCmdLine);
      return;
    }
    var param = aCmdLine.handleFlagWithParam("printpdf", false);
    if (!param) {
      dump("Missing output file name.\n");
      flushCmdLine(aCmdLine);
      return;
    }
    var outputURI = aCmdLine.resolveURI(param);
    if (!(outputURI instanceof Components.interfaces.nsIFileURL)) {
      dump("Output is not a local file.\n");
      flushCmdLine(aCmdLine);
      return;
    }

    var argcount = aCmdLine.length;
    switch (argcount) {
    case 0:
      dump("Missing URL argument.\n");
      return;
    default:
      dump("Extra arguments ignored.\n");
    case 1:
    }

    var urlarg = aCmdLine.getArgument(0);
    flushCmdLine(aCmdLine);

    var uri = aCmdLine.resolveURI(urlarg);
    if ((uri instanceof Components.interfaces.nsIFileURL)
      && !uri.file.exists()) {
      dump("Local file " + uri.file.path + " does not exists.\n");
      return;
    }
    if (isDisabled()) {
      dump("PrintPDF extension is disabled.\n");
      return;
    }
    if (uri)
      openWindow(null, "chrome://printpdf/content/main.xul", "_blank",
        "chrome,dialog=no,all", [uri.spec, outputURI.file.path]);
  },

  helpInfo : "  -printpdf <path>   Print to this PDF file.\n",

  classDescription: "PrintPDFHandler",
  classID: Components.ID("{5931a4f2-b045-4f02-877f-69184ce00bf9}"),
  contractID: "@jasp.net/iceweasel/printpdf;1",
  QueryInterface:
    XPCOMUtils.generateQI([Components.interfaces.nsICommandLineHandler]),
  _xpcom_categories: [{category: "command-line-handler", entry: "m-printpdf"}]
};

function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule([PrintPDFHandler]);
}
